%% Evolutionary Algorithm framework
function [ea] = EA(config, algo, varargin)

global state

ea.config ='';
ea.algo = '';

ea.param_value = [];

ea.prob = [];
ea.object = [];
ea.param = [];
ea.analysis = [];
ea.pop = [];
ea.best = [];
ea.surr = [];

ea = class(ea, 'EA');

% State information
state = [];
state.gen_id = 0;
state.ind_id = 0;

if nargin >= 2
	if isa(config, 'function_handle')
		ea.config = func2str(config);
	else
		ea.config = config;
	end
	ea.algo = algo;

	if nargin == 3, ea.param_value = varargin{1}; end

	params = load_params(ea);
	ea.param = Paramset(params, ea.param_value);
	
	% set random number state
	if ea.param.seed == 0
		ea.param = set(ea.param, 'seed', round(rand(1)*2^32));
	end
	rand('twister', ea.param.seed);

	% For structural problems loading problem can be time consuming
	%  Cache problem definitions as matlab arrays
	cachefile = sprintf('%s-cache.mat', ea.config);
	try
		tmp = load(cachefile);
		prob = tmp.prob;
	catch
		% load problem definition
		prob = feval(config);
		if ~isfield(prob, 'class')
			prob.class = 'Numeric';
		end
		if strcmp(prob.class, 'Solid2d') || strcmp(prob.class, 'Solid3d')
			save(cachefile, 'prob');
		end
	end
	ea.prob = prob;
	
	% Create object
	ea.object = feval(ea.prob.class, ea.prob, ea.param);

	% Analysis function
	if isfield(ea.prob, 'analysis')
		ea.analysis = Analysis(ea.prob.analysis);
	else
		if strcmp(prob.class, 'Numeric')
			ea.analysis = Analysis(config, ea.param_value);
		elseif strcmp(prob.class, 'Truss2d') ...
			|| strcmp(prob.class, 'Truss3d') ...
			|| strcmp(prob.class, 'Solid2d') ...
			|| strcmp(prob.class, 'Solid3d')
			ea.analysis = config;
		else
			error('No Analysis found');
		end
	end
	
	% Run the EA
	if nargin == 2 && ~strcmp(ea.algo, 'eval')
		ea = run(ea);
	end
end

return
