function [ea] = IDEA_MA4(ea)
% Infeasibility Driven Evolutionary Algorithm

global logger state

N = ea.param.pop_size;
assert(mod(N,4) == 0, 'Population size should be multiple of 4.');

write(logger, 'Starting IDEA\n');

% Generation 1
state.gen_id = 1;
pop = Population(ea.object, N, ea.prob);
pop = init(pop);
pop = eval(pop, ea.analysis);
f_new = new_objective(pop, ea.param.infeasible_strategy);
pop = sort_idea(pop, ea.param.infeasible_ratio, f_new);
ea = stats(ea, pop, 'ini');
f_best2 = pop.f(find(pop.feas==1,1));

range = convert_range(pop.object);
min_x = zeros(1,ea.prob.nx);
max_x = zeros(1,ea.prob.nx);
for i = 1:ea.prob.nx
	min_x(i) = min(range{i});
	max_x(i) = max(range{i});
end

% Generations 2-N

change = 0;
change_all = 0;
for i = 2:ea.param.generations
	state.gen_id = i;
    if rand <0.5
        childpop = evolve(pop);
    else
        childpop = evolve_de(pop,ea.prob);
    end

	childpop = eval(childpop, ea.analysis);
	pop = pop + childpop;
	f_new = new_objective(pop, ea.param.infeasible_strategy);
	pop = sort_idea(pop, ea.param.infeasible_ratio, f_new);
	pop = reduce(pop, N);

    
    if sum(pop.feas) > 0
        f_best1 = pop.f(find(pop.feas==1,1)); % f_best before going into local search
    end    
    
    t1 = rand;
    if t1 < 0.2 
        start_x = randint(1,1,[1 pop.size]);
    else     

        if f_best1 < f_best2
            start_x =1;
        else
            if change == 0         
               if sum(pop.feas)>0
                    idx = find(pop.feas==1);
                    start_x = randint(1,1,[1 idx(end)]);
               else
                   start_x = randint(1,1,[1 ea.param.infeasible_ratio*pop.size]);
               end
            else
                if sum(pop.feas)>0
                    start_x = find(pop.feas==1,1);
                else
                    start_x = 1;
                end
            end
        end
    end


        
%         if ea.param.max_fn_evals - ea.pop.fn_evals < 50000
%             start_x = 1;
%         end
    
        [pop,change] = do_local_search(start_x,ea,pop,min_x,max_x,f_best1);
        f_best2 = pop.f(find(pop.feas ==1,1)); % This is the pop that goes into next generation for evaluation
        change_all = [change_all change];
       
        
        if length(change_all) > 10
            disp(sum(change_all(end-9:end)))
            if sum(change_all(end-9:end)) ==0
                fprintf('Reinitializing pop\n');
                [pop,f_best2] = reinitialize_pop(pop,ea);
                change_all = [];
%                 keyboard
            end
        end
        
    
    ea = stats(ea, pop, 'gen');
	if ea.param.max_fn_evals > 0 && pop.fn_evals >= ea.param.max_fn_evals
		break
    end
end

ea.pop = pop;
ea = stats(ea, pop, 'fin');

return

function [pop,f_best2] = reinitialize_pop(pop,ea)

tmp = pop;
pop = Population(ea.object, ea.param.pop_size, ea.prob);
pop = init(pop);

for i = 1
    pop.x{i} = tmp.x{i};
end

pop = eval(pop,ea.analysis);
f_new = new_objective(pop, ea.param.infeasible_strategy);
pop = sort_idea(pop, ea.param.infeasible_ratio, f_new);
pop = reduce(pop,ea.param.pop_size);
f_best2 = pop.f(1);
pop.fn_evals = pop.fn_evals + tmp.fn_evals;

return

% local search
function [pop,change] = do_local_search(start_x,ea,pop,min_x,max_x,f_best)

%     if pop.feas(start_x)==0
%         change = 0;
%         return;
%     end
    
    x1 = convert_x(pop.object, pop.x{start_x});
%     start_x
%     pause
    n_var = length(x1);
	warning off all;
    options = optimset('Display', 'off', 'MaxFunEvals',max(1000,50*n_var),'MaxIter',10000,'TolCon',1e-6,'TolFun',1e-6);
 	[x2,f2,eflag,output] = fmincon(@(x) obj_func(x,ea,1), x1, ...
 		[], [], [], [], min_x, max_x, ...
 		[], options);

%     disp([ x1 pop.f(1) -pop.g(1,:)]);
    [f,g] = obj_func(x2,ea,3);
%     disp([ x2 f g ]);

	if pop.feas(start_x) == 1 && (sum(g>0)>0)
		replace_flag=0;
	else
		replace_flag = 1;
	end
	
    if(replace_flag==1)
		pop.x{end}=convert_obj(pop.object,x2);
        pop.f(end)=f;
		pop.g(end,:)=-g;
        
        if(sum(g>0)==0)
            pop.feas(end)=1;
        else
            pop.feas(end)=0;
        end
        
		f_new = new_objective(pop, ea.param.infeasible_strategy);
		pop = sort_idea(pop, ea.param.infeasible_ratio,f_new);
        pop = reduce(pop,ea.param.pop_size);
        
%         fprintf('Start f : %d\n',tp1(x1));
%         fprintf('End f : %d\n',f2);        
        
        if f < f_best
            change = 1;
        else 
            change = 0;
        end
    else
        change = 0;
    end
    

	pop.fn_evals=pop.fn_evals+output.funcCount;
return


%% NSGA-II evolution strategy
function [childpop] = evolve(pop)
N = pop.size;
childpop = Population(pop, N);
a1 = randperm(N);
a2 = randperm(N);

for i = 1:4:N
	p1 = tournament(pop, a1(i), a1(i+1));
	p2 = tournament(pop, a1(i+2), a1(i+3));
	[c1, c2] = crossover(pop, 'SBX', p1, p2);
	childpop.x{i} = c1;
	childpop.x{i+1} = c2;
	p1 = tournament(pop, a2(i), a2(i+1));
	p2 = tournament(pop, a2(i+2), a2(i+3));
	[c1, c2] = crossover(pop, 'SBX', p1, p2);
	childpop.x{i+2,:} = c1;
	childpop.x{i+3,:} = c2;
end

% Mutation
for i = 1:childpop.size
	childpop.x{i} = mutation(childpop, 'POLY', childpop.x{i});
end
	
return


%% additional objective
function [f_new] = new_objective(pop, strategy)

if strategy == 1
	% number of constraint violations
	f_new = sum(pop.g < 0, 2);
elseif strategy == 2
	% constraint violation measure (rank based)
	tmp = [pop.g < 0];
	c = pop.g .* tmp;
	con_f = zeros(pop.size, pop.ng);

	for j = 1:pop.ng
		[tmp, I] = sort(c(:,j), 'descend');
		cur_rank = 0;
		cur_val = 0;
		for i = 1:pop.size
			if tmp(i) < cur_val
				cur_rank = cur_rank+1;
				cur_val = tmp(i);
			end
			con_f(I(i), j) = cur_rank;
		end
	end

	f_new = sum(con_f, 2);
else
	error('IDEA: Unknown strategy');
end

return
%% objective function
function [obj1,obj2] = obj_func(x,ea,flag_fg)
% x
sfunc1=ea.config;
[f g]=feval(sfunc1,x);

if(flag_fg==1)
	obj1=f;
	obj2=[];
end
if(flag_fg==2)
	obj1=-g;
	obj2=[];
end
if(flag_fg==3)
	obj1=f;
	obj2=-g;
end

return

%% DE evolution strategy
function [childpop] = evolve_de(pop,prob)

%DE Control parameters
% TODO: read from params.m
    F=0.9;Cr=0.9;
    
    N = pop.size;
    childpop = Population(pop, N);

    for i = 1:N
        
        % current solution
        x_i=convert_x(pop.object,pop.x{i});
        c=x_i;
        
        %% create trial vector
        
        % pick up random p1,p2,p3
        p1=1;p2=1;p3=1;
        while p1~=p2~=p3
            p1=randi(N);p2=randi(N);p3=randi(N);
        end       

        x_p1=convert_x(pop.object,pop.x{p1});
        x_p2=convert_x(pop.object,pop.x{p2});
        x_p3=convert_x(pop.object,pop.x{p3});        
        
        % trial vector
        v=x_p1+F*(x_p2-x_p3);
        
        %% Exponential crossover

        %Find n,L for crossover
        D=length(x_p1); n=randi(D);
        L=0;
        while 1
            L=L+1;
            if rand > Cr || L > D
                break
            end
        end
        
        for j=mod(n,D):mod(n+L-1,D)
            if j==0
                continue;
            end
            c(j)=v(j);
        end
        
	childpop.x{i} = convert_obj(pop.object,c);
%     pop.x{end}=convert_obj(pop.object,x2);
        
        % Polynomial mutation
        childpop.x{i} = mutation(childpop, 'POLY', childpop.x{i});
        
        % Bounds check
        x_c=convert_x(pop.object,childpop.x{i});
        tmp2=0;
        for j=1:prob.nx
            bounds=prob.range{j};
            if x_c(j) < min(bounds)
                x_c(j) = min(bounds);
            end
            if x_c(j) > max(bounds)
                x_c(j) = max(bounds);
            end
        end

	childpop.x{i} =convert_obj(pop.object,x_c); 
    
    
    end
    
    return
    
