%% Collect statistics and Show summary
function [ea] = stats(ea, pop, flag)

global state

if isa(pop, 'Population')
	ea = stats_pop(ea, pop, flag, state.gen_id);
elseif isa(pop, 'Individual')
	ea = stats_ind(ea, pop, flag, state.gen_id);
end
if ea.param.batch_mode ~= 1
	ea.pop = pop;
	plot(ea);
end
return


%% Population stats
function [ea] = stats_pop(ea, pop, flag, gen)

switch flag
	case 'ini'
		save_pop(ea, pop, gen, 'initial', 'w');
		save_pop(ea, pop, gen, 'all', 'w');
		ea = update_best_pop(ea, pop, gen);
		summary(pop, gen);
	case 'gen'
		save_pop(ea, pop, gen, 'all', 'a');
		ea = update_best_pop(ea, pop, gen);
		summary(pop, gen);
	case 'fin'
		save_pop(ea, pop, gen, 'final', 'w');
		save_best(ea);
	otherwise
		error('Unknown flag (%s)', flag);
end
return


function [ea] = update_best_pop(ea, pop, n)
id = find(pop.eval == 1 & pop.feas == 1, 1);
if isempty(id)
	id = find(pop.nd_rank == 1, 1);
	if isempty(id)
		id = pop.rank(1);
	end
end

ea.best.x(n,:) = pop.x(id,:);
ea.best.f(n,:) = pop.f(id,:);
if pop.ng > 0
	ea.best.g(n,1:pop.ng) = pop.g(id,:);
end
ea.best.feas(n) = pop.feas(id);
ea.best.fn_evals(n) = pop.fn_evals;
return


%% Individual stats
function [ea] = stats_ind(ea, ind, flag, gen)

switch flag
	case 'ini'
		save_pop(ea, ind, gen, 'all', 'w');
		ea = update_best_ind(ea, ind, gen);
		summary(ind, gen);
	case 'gen'
		save_pop(ea, ind, gen, 'all', 'a');
		ea = update_best_ind(ea, ind, gen);
		summary(ind, gen);
	case 'fin'
		save_pop(ea, ind, gen, 'final', 'w');
	otherwise
		error('Unknown flag (%s)', flag);
end
return


function [ea] = update_best_ind(ea, ind, n)
ea.best.f(n,:) = ind.f;
if ind.ng > 0
	ea.best.g(n,:) = ind.g;
end
ea.best.feas(n) = 1;
return
